#pragma once

#include <mutex>

#include <ArmarXCore/core/ArmarXManager.h>
#include <ArmarXCore/core/Component.h>
#include <ArmarXCore/core/ManagedIceObjectRegistryInterface.h>
#include <ArmarXCore/core/application/Application.h>

#include <RobotAPI/components/ArViz/ArVizStorage.h>

#include <arviz_godot/lib/ArVizInteractiveInterface.h>
#include <arviz_godot/lib/avi/app/ApplicationSettings.h>
#include <arviz_godot/lib/avi/connection/Component.h>

namespace avi::app
{
    class ArVizInteractive :
        virtual public armarx::Component,
        virtual public avi::connection::Component,
        virtual public arviz_interactive_project::components::arviz_interactive::
            ArVizInteractiveInterface
    {
    public:

        ~ArVizInteractive() override = default;

        std::string getDefaultName() const override;

        static ArVizInteractive* Initialize(avi::app::ApplicationSettings& settings);

        bool isConnected() const override;

        armarx::viz::StorageInterfacePrx getStorageInterface() const override;

        void close();

    protected:
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;

        void onInitComponent() override;
        void onConnectComponent() override;
        void onDisconnectComponent() override;
        void onExitComponent() override;

    private:
        struct ArVizInteractiveApp : public armarx::Application
        {
        public:
            void setup(const armarx::ManagedIceObjectRegistryInterfacePtr& registry,
                       Ice::PropertiesPtr properties) override;

        public:
            std::mutex mutex{};
            std::condition_variable condition{};
            std::thread thread;

            bool setupDone{false};
            IceInternal::Handle<avi::app::ArVizInteractive> component{};

            int exec(const armarx::ArmarXManagerPtr& armarXManager) override;
        };

        static std::string storageName;
        static std::string defaultName;

        armarx::viz::StorageInterfacePrx storage{};
        IceUtil::Handle<ArVizInteractiveApp> app{};

        bool connected{false};

        bool closed{false};
        std::condition_variable closedCondition{};
        std::mutex closedMutex{};
    };
} // namespace avi::app
