#include "RevisionLoadCycle.h"

#include <chrono>

avi::app::RevisionLoadCycle::RevisionLoadCycle(avi::connection::Component& component,
                                               avi::connection::RevisionBuffer& revisions,
                                               const avi::core::SettingsProvider& settings) :
    component(component), revisions(revisions), settings(settings)
{
}

void
avi::app::RevisionLoadCycle::run(bool mockStorage)
{
    if (mockStorage)
    {
        storage = std::make_unique<avi::connection::MockStorage>();
    }
    else
    {
        storage = std::make_unique<avi::connection::ElementStorage>(component);
    }

    running = true;

    cycle = std::thread(
        [this]()
        {
            double freq = settings.getRevisionCycleFrequency();
            std::chrono::milliseconds interval(static_cast<int>(1000.0 / freq));

            while (running)
            {
                std::chrono::steady_clock::time_point start = std::chrono::steady_clock::now();

                armarx::viz::data::LayerUpdates revision = storage->getNext();
                revisions.addUpdates(std::move(revision));

                std::chrono::steady_clock::time_point end = std::chrono::steady_clock::now();

                std::chrono::milliseconds elapsed =
                    std::chrono::duration_cast<std::chrono::milliseconds>(end - start);
                std::chrono::milliseconds remaining = interval - elapsed;

                if (remaining.count() > 0)
                    std::this_thread::sleep_for(remaining);
            }
        });
}

avi::connection::ElementStorage&
avi::app::RevisionLoadCycle::getStorage()
{
    return *storage;
}

void
avi::app::RevisionLoadCycle::stop()
{
    running = false;
    cycle.join();
}
