#include "ElementStorage.h"

#include <arviz_godot/lib/avi/connection/LiveStorageAdapter.h>
#include <arviz_godot/lib/avi/connection/ReplayStorageAdapter.h>

avi::connection::ElementStorage::ElementStorage(avi::connection::Component& component) :
    component(component), storage(std::make_unique<LiveStorageAdapter>(component))
{
}

armarx::viz::data::LayerUpdates
avi::connection::ElementStorage::getNext()
{
    if (switchToLive)
    {
        storage = std::make_unique<LiveStorageAdapter>(component);
        switchToLive = false;
    }
    else if (switchToReplay)
    {
        storage = std::make_unique<ReplayStorageAdapter>(component, clock, replay);
        storage->setup();
        switchToReplay = false;
    }

    std::lock_guard<std::mutex> lock(feedbackMutex);

    storage->setInteraction(feedbackVec);
    feedbackVec = {};

    return storage->getLayerUpdates();
}

bool
avi::connection::ElementStorage::isConnected() const
{
    return component.isConnected();
}

bool
avi::connection::ElementStorage::isLive() const
{
    return live;
}

bool
avi::connection::ElementStorage::isRecording() const
{
    return recording;
}

void
avi::connection::ElementStorage::recordReplay(const std::string& name)
{
    recording = true;
    component.getStorageInterface()->startRecording(name);
}

void
avi::connection::ElementStorage::endRecording()
{
    recording = false;
    component.getStorageInterface()->stopRecording();
}

std::vector<std::string>
avi::connection::ElementStorage::getAllReplays()
{
    return storage->getAllReplays();
}

void
avi::connection::ElementStorage::useLiveTiming()
{
    live = true;
    switchToLive = true;
}

void
avi::connection::ElementStorage::useReplayTiming(const std::string& replay,
                                                 std::shared_ptr<avi::connection::Clock> clock)
{
    live = false;
    switchToReplay = true;

    this->replay = replay;
    this->clock = clock;
}

void
avi::connection::ElementStorage::addInteraction(avi::connection::Interaction interaction)
{
    std::lock_guard<std::mutex> lock(feedbackMutex);
    feedbackVec.push_back(interaction);
}
