#include "MockStorage.h"

avi::connection::MockStorage::MockStorage() : ElementStorage(this->component)
{
}

uint8_t x = 0;

armarx::viz::data::LayerUpdates
avi::connection::MockStorage::getNext()
{
    x += 1;

    if (x == 200)
        lSwitch = !lSwitch;

    armarx::viz::data::LayerUpdates layerUpdates;

    armarx::viz::data::LayerUpdate layerUpdate;
    layerUpdate.component = "Component";
    layerUpdate.name = "Layer 1";
    layerUpdate.action = armarx::viz::data::Layer_CREATE_OR_UPDATE;

    armarx::viz::data::LayerUpdate layerUpdate2;
    layerUpdate2.component = "Component";
    layerUpdate2.name = "Layer 2";
    layerUpdate2.action =
        lSwitch ? armarx::viz::data::Layer_CREATE_OR_UPDATE : armarx::viz::data::Layer_DELETE;

    IceUtil::Handle<armarx::viz::data::ElementBox> element(new armarx::viz::data::ElementBox());
    element->id = "Box1";
    element->size = {100, 100, 100};
    element->pose = {0, 100, 0, 1, 0, 0, 0};
    setStandardParameter(element, element->id);
    layerUpdate.elements.push_back(element);

    IceUtil::Handle<armarx::viz::data::ElementBox> element2(new armarx::viz::data::ElementBox());
    element2->id = "Box2";
    element2->size = {100, 100, 100};
    element2->pose = {0, -100, 0, 1, 0, 0, 0};
    setStandardParameter(element2, element2->id);
    layerUpdate2.elements.push_back(element2);

    layerUpdates.updates.push_back(layerUpdate);
    layerUpdates.updates.push_back(layerUpdate2);

    return layerUpdates;
}

bool
avi::connection::MockStorage::isConnected() const
{
    return true;
}

bool
avi::connection::MockStorage::isLive() const
{
    return live;
}

void
avi::connection::MockStorage::recordReplay(const std::string& name)
{
    std::cout << "MockStorage::recordReplay(" << name << ")" << std::endl;
}

void
avi::connection::MockStorage::endRecording()
{
    std::cout << "MockStorage::endRecording()" << std::endl;
}

std::vector<std::string>
avi::connection::MockStorage::getAllReplays()
{
    std::vector<std::string> result;
    result.emplace_back("replay");
    return result;
}

void
avi::connection::MockStorage::useLiveTiming()
{
    live = true;

    std::cout << "MockStorage::useLiveTiming()" << std::endl;
}

void
avi::connection::MockStorage::useReplayTiming(const std::string& replay,
                                              std::shared_ptr<avi::connection::Clock> clock)
{
    live = false;

    std::cout << "MockStorage::useReplayTiming(" << replay << ")" << std::endl;
}

void
avi::connection::MockStorage::addInteraction(avi::connection::Interaction interaction)
{
    std::cout << "MockStorage::addInteraction(" << interaction.getElement()->getName() << ")"
              << std::endl;
    std::cout << "interaction =" << std::endl;
    std::cout << "type:        " << std::to_string((int)interaction.getType()) << std::endl;
    std::cout << "position:    "
              << "(" << std::to_string(interaction.getTranslation().x) << ", "
              << std::to_string(interaction.getTranslation().y) << ", "
              << std::to_string(interaction.getTranslation().z) << ")" << std::endl;
    std::cout << "transform:   " << std::to_string((int)interaction.getTransformation())
              << std::endl;
}

void
avi::connection::MockStorage::setStandardParameter(
    IceUtil::Handle<armarx::viz::data::Element> element,
    std::basic_string<char> id)
{
    element->scale = {1, 1, 1};
    //element->color = {255, 255, 0, 0};
    element->interaction.contextMenuOptions = {id,
                                               "flags:",
                                               "SELECT",
                                               "CONTEXT_MENU",
                                               "TRANSLATION_X",
                                               "implicit global manipulation and transform show"};
    element->interaction.enableFlags = element->interaction.enableFlags |
                                       armarx::viz::data::InteractionEnableFlags::SELECT |
                                       armarx::viz::data::InteractionEnableFlags::CONTEXT_MENU |
                                       armarx::viz::data::InteractionEnableFlags::TRANSLATION_X |
                                       armarx::viz::data::InteractionEnableFlags::TRANSLATION_Y |
                                       armarx::viz::data::InteractionEnableFlags::TRANSLATION_Z |
                                       armarx::viz::data::InteractionEnableFlags::SCALING_X |
                                       armarx::viz::data::InteractionEnableFlags::SCALING_Y |
                                       armarx::viz::data::InteractionEnableFlags::SCALING_Z |
                                       armarx::viz::data::InteractionEnableFlags::ROTATION_X |
                                       armarx::viz::data::InteractionEnableFlags::ROTATION_Y |
                                       armarx::viz::data::InteractionEnableFlags::ROTATION_Z;
}

bool
avi::connection::MockStorage::MockComponent::isConnected() const
{
    return true;
}

armarx::viz::StorageInterfacePrx
avi::connection::MockStorage::MockComponent::getStorageInterface() const
{
    return {};
}
