#pragma once

#include <arviz_godot/lib/avi/connection/Clock.h>
#include <arviz_godot/lib/avi/connection/Component.h>
#include <arviz_godot/lib/avi/connection/StorageAdapter.h>

namespace avi::connection
{
    class ReplayStorageAdapter : public avi::connection::StorageAdapter
    {
    public:
        void setInteraction(std::vector<avi::connection::Interaction> newInteractions) override;

        explicit ReplayStorageAdapter(avi::connection::Component& component,
                                      std::shared_ptr<avi::connection::Clock> clock,
                                      std::string replay);

        ~ReplayStorageAdapter() override = default;

        armarx::viz::data::LayerUpdates getLayerUpdates() override;

        std::vector<std::string> getAllReplays() override;

        void setup() override;

    private:
        avi::connection::Component& component;

        std::shared_ptr<avi::connection::Clock> clock;

        std::string activeReplay;

        std::map<int64_t, armarx::viz::data::RecordingBatch> batchCache{};

        std::map<int64_t, int64_t> batches{};

        static size_t
        binarySearch(const std::vector<armarx::viz::data::TimestampedLayerUpdate>& updates,
                     size_t l,
                     size_t r,
                     int64_t timeStamp);

        std::chrono::steady_clock::time_point last = std::chrono::steady_clock::now();

        const static int64_t BATCHES_FRONT = 3;

        const static int64_t BATCHES_BACK = 1;

        const static int64_t BATCH_CACHE_SIZE = BATCHES_FRONT + BATCHES_BACK + 1;
    };
} // namespace avi::connection
