#pragma once

#include <string>

#define ADD_OPTION(TYPE, VALUE, NAME)                                                              \
    emplace_back(std::make_unique<avi::core::ValueOption<TYPE, T>>(#VALUE, NAME, VALUE))

namespace avi::core
{
    template <typename T>
    class Option
    {
    protected:
        explicit Option(std::string key, std::string label);

    public:
        virtual void accept(T& transformer) = 0;

        /// Get the key used in storage to refer to this option.
        /// The key must be unique and a valid identifier.
        [[nodiscard]] const std::string& getKey() const;

        /// Get the label referring to this option.
        /// The label must be human-readable.
        [[nodiscard]] const std::string& getLabel() const;

        virtual ~Option() = default;

    private:
        std::string key;
        std::string label;
    };
} // namespace avi::core

#include "Option.tpp"
