#pragma once

#include <Godot.hpp>
#include <arviz_godot/lib/avi/core/OptionFile.h>
#include <arviz_godot/lib/avi/core/ValueOption.h>

namespace avi::core
{
    template <typename O>
    class OptionFileWriter
    {
    public:
        explicit OptionFileWriter(O& optionContainer);

        void write(avi::core::OptionFile& optionFile);

        template <typename V>
        void transform(avi::core::ValueOption<V, OptionFileWriter>& option);

    private:
        template <typename V>
        static void
        transform(avi::core::ValueOption<V, OptionFileWriter>& option,
                  avi::core::OptionFile& optionFile)
        {
            auto v = option.getValue();
            optionFile.template setValue<V>(option.getKey(), v);
        }

    private:
        O& optionContainer;
        avi::core::OptionFile* currentOptionFile{};
    };

    template <typename O>
    OptionFileWriter<O>::OptionFileWriter(O& optionContainer) : optionContainer(optionContainer)
    {
    }

    template <typename O>
    void
    OptionFileWriter<O>::write(OptionFile& optionFile)
    {
        currentOptionFile = &optionFile;

        for (auto& option : optionContainer.template getOptions<OptionFileWriter<O>>())
        {
            option->accept(*this);
        }

        currentOptionFile = nullptr;
    }

    template <typename O>
    template <typename V>
    void
    OptionFileWriter<O>::transform(ValueOption<V, OptionFileWriter>& option)
    {
        transform(option, *currentOptionFile);
    }
} // namespace avi::core
