#include "Settings.h"

#include <iostream>
#include <utility>

avi::core::Settings::Settings(std::filesystem::path path) : path(std::move(path))
{
    optionFile = avi::core::OptionFile::load(this->path);

    this->storageIdentifier = "ArVizStorage";
    this->orbitSensitivityTechnical = 100;
    this->zoomSensitivityTechnical = 100;
    this->rollSensitivityTechnical = 100;
    this->mouseSensitivityPlayful = 100;
    this->rollSensitivityPlayful = 100;
    this->moveSpeedPlayful = 100;
    this->revisionCycleFrequency = 60.0;
    this->defaultProfileName = "Default";
    this->profileDirectory = this->path.parent_path() / "profiles";
    this->screenshotDirectory = getPath("XDG_PICTURES_DIR", "Pictures");
    this->videoDirectory = getPath("XDG_VIDEOS_DIR", "Videos");

    if (!optionFile.has_value())
    {
        optionFile = avi::core::OptionFile::create("settings");
    }
    else
    {
        avi::core::OptionFileReader<Settings> reader(*this);
        reader.read(optionFile.value());
    }
}

void
avi::core::Settings::save()
{
    avi::core::OptionFileWriter<Settings> writer(*this);
    writer.write(optionFile.value());
    optionFile->save(path);
}

avi::core::Settings
avi::core::Settings::getDefaultSettings()
{
    std::filesystem::path aviConfigDirectory = getPath("XDG_CONFIG_HOME", ".config");
    std::filesystem::create_directories(aviConfigDirectory);

    return avi::core::Settings(aviConfigDirectory / "settings.opt");
}

std::filesystem::path
avi::core::Settings::getPath(const char* xdg, const std::string& directoryInHome)
{
    char* xdgPath = std::getenv(xdg);

    std::filesystem::path directory =
        xdgPath ? std::filesystem::path(xdgPath)
                : (std::filesystem::path(std::getenv("HOME")) / directoryInHome);

    return directory / "avi";
}
