#pragma once

#include <filesystem>
#include <vector>

#include <arviz_godot/lib/avi/core/OptionFileReader.h>
#include <arviz_godot/lib/avi/core/OptionFileWriter.h>

namespace avi::core
{
    struct Settings
    {
        void save();

        static avi::core::Settings getDefaultSettings();

        std::string storageIdentifier{};
        double orbitSensitivityTechnical{};
        double zoomSensitivityTechnical{};
        double rollSensitivityTechnical{};
        double mouseSensitivityPlayful{};
        double moveSpeedPlayful{};
        double rollSensitivityPlayful{};
        double revisionCycleFrequency{};
        std::string defaultProfileName{};
        std::filesystem::path profileDirectory{};
        std::filesystem::path screenshotDirectory{};
        std::filesystem::path videoDirectory{};

        template <typename T>
        auto
        getOptions()
        {
            std::vector<std::unique_ptr<avi::core::Option<T>>> options;

            options.ADD_OPTION(std::string, storageIdentifier, "Storage Identifier");
            options.ADD_OPTION(double, orbitSensitivityTechnical, "[Technical] Orbit Sensitivity");
            options.ADD_OPTION(double, zoomSensitivityTechnical, "[Technical] Zoom Sensitivity");
            options.ADD_OPTION(double, rollSensitivityTechnical, "[Technical] Roll Sensitivity");
            options.ADD_OPTION(double, mouseSensitivityPlayful, "[Playful] Mouse Sensitivity");
            options.ADD_OPTION(double, rollSensitivityPlayful, "[Playful] Roll Sensitivity");
            options.ADD_OPTION(double, moveSpeedPlayful, "[Playful] Move Speed");
            options.ADD_OPTION(double, revisionCycleFrequency, "Revision Cycle Frequency");
            options.ADD_OPTION(std::string, defaultProfileName, "Default Profile Name");
            options.ADD_OPTION(std::filesystem::path, profileDirectory, "Profile Directory");
            options.ADD_OPTION(std::filesystem::path, screenshotDirectory, "Screenshot Directory");
            options.ADD_OPTION(std::filesystem::path, videoDirectory, "Video Directory");

            return options;
        }

    private:
        explicit Settings(std::filesystem::path path);

        static std::filesystem::path getPath(const char* xdg, const std::string& directoryInHome);

    private:
        std::filesystem::path path;
        std::optional<avi::core::OptionFile> optionFile{};
    };
} // namespace avi::core
