#pragma once

#include <functional>

#include <Control.hpp>
#include <Godot.hpp>
#include <LineEdit.hpp>
#include <OptionButton.hpp>
#include <arviz_godot/lib/avi/gui/VisualOptionTransformer.h>
#include <arviz_godot/lib/avi/scene/GraphicsController.h>
#include <arviz_godot/lib/avi/scene/ProfileStorage.h>

namespace avi::gui
{
    class GraphicsSettings : public godot::Control
    {
        GODOT_CLASS(GraphicsSettings, Control)

    public:
        virtual ~GraphicsSettings() = default;

        static void _register_methods();

        void _init();

        void _ready();

        void setup(avi::scene::GraphicsController& controller, avi::scene::ProfileStorage& storage);

    private:
        void _save();

        void _copy();

        void _select(int64_t idx);

        void _change(godot::Variant v);

        void update();

        void setName(const std::string& name);

        std::string getName();

        void setSavedFlag(bool flag);

    private:
        avi::scene::GraphicsController* controller{};
        avi::scene::ProfileStorage* storage{};

        godot::OptionButton* options{};
        godot::Button* saveButton{};
        godot::LineEdit* nameBox{};

        godot::Control* container{};
        std::vector<std::unique_ptr<avi::core::Option<avi::gui::VisualOptionTransformer>>>
            optionStore{};
        std::vector<std::function<void()>> setters{};
    };
} // namespace avi::gui
