#include "ManipulationMenu.h"

#include <InputEvent.hpp>

void
avi::gui::ManipulationMenu::_register_methods()
{
    godot::register_method("_ready", &avi::gui::ManipulationMenu::_ready);
    godot::register_method("_process", &avi::gui::ManipulationMenu::_process);
}

void
avi::gui::ManipulationMenu::_init()
{
}

void
avi::gui::ManipulationMenu::setup(avi::interaction::Manipulator& manipulator,
                                  avi::core::EventSubscriptionSystem& eventSubscriptionSystem)
{
    this->manipulator = &manipulator;

    eventSubscriptionSystem.subscribe("MANIPULATOR_NONE",
                                      [this]() -> void { this->select(MANIPULATOR_NONE); });
    eventSubscriptionSystem.subscribe("MANIPULATOR_MOVE",
                                      [this]() -> void { this->select(MANIPULATOR_MOVE); });
    eventSubscriptionSystem.subscribe("MANIPULATOR_ROTATE",
                                      [this]() -> void { this->select(MANIPULATOR_ROTATE); });
    eventSubscriptionSystem.subscribe("MANIPULATOR_SCALE",
                                      [this]() -> void { this->select(MANIPULATOR_SCALE); });

    this->set_item_text(MANIPULATOR_NONE, "[3]");
    this->set_item_text(MANIPULATOR_MOVE, "Move [4]");
    this->set_item_text(MANIPULATOR_ROTATE, "Rotate [5]");
    this->set_item_text(MANIPULATOR_SCALE, "Scale [6]");
}

void
avi::gui::ManipulationMenu::_process()
{
    setActiveManipulator();
}

void
avi::gui::ManipulationMenu::_ready()
{
    this->manipulatorCount = this->get_item_count();
}

int
avi::gui::ManipulationMenu::getSelectedItemID()
{
    for (int i = 0; i < manipulatorCount; i++)
    {
        if (this->is_selected(i))
        {
            return i;
        }
    }

    return -1;
}

void
avi::gui::ManipulationMenu::setActiveManipulator()
{
    int activeManipulatorID = getSelectedItemID();

    switch (activeManipulatorID)
    {
        case MANIPULATOR_NONE:
            avi::gui::ManipulationMenu::disableActiveItemText();
            selectionMode = ManipulationMode::NONE;
            manipulator->setManipulationMode(selectionMode);
            break;
        case MANIPULATOR_MOVE:
            avi::gui::ManipulationMenu::disableActiveItemText();
            selectionMode = ManipulationMode::TRANSLATE;
            manipulator->setManipulationMode(selectionMode);
            break;
        case MANIPULATOR_ROTATE:
            avi::gui::ManipulationMenu::disableActiveItemText();
            selectionMode = ManipulationMode::ROTATE;
            manipulator->setManipulationMode(selectionMode);
            break;
        case MANIPULATOR_SCALE:
            avi::gui::ManipulationMenu::disableActiveItemText();
            selectionMode = ManipulationMode::SCALE;
            manipulator->setManipulationMode(selectionMode);
            break;
        default:
            break;
    }
}

void
avi::gui::ManipulationMenu::disableActiveItemText()
{
}
