#include "ReplayCockpit.h"

#include <iostream>

#include "UserInterface.h"
#include <arviz_godot/lib/avi/scene/VisualizedScene.h>

void
avi::gui::ReplayCockpit::_register_methods()
{
    register_method("_ready", &ReplayCockpit::_ready);

    register_method("_process", &ReplayCockpit::_process);

    register_method("_record", &ReplayCockpit::_record);

    register_method("_enterReplayMode", &ReplayCockpit::_enterReplayMode);

    register_method("_select", &ReplayCockpit::_select);
    register_method("_refresh", &ReplayCockpit::update_replays);

    register_method("_play", &ReplayCockpit::_play);

    register_method("_slide", &ReplayCockpit::_slide);

    register_method("_forward1", &ReplayCockpit::_forward1);
    register_method("_forward2", &ReplayCockpit::_forward2);
    register_method("_rewind1", &ReplayCockpit::_rewind1);
    register_method("_rewind2", &ReplayCockpit::_rewind2);

    register_method("_useSlider", &ReplayCockpit::_useSlider);
    register_method("_unUseSlider", &ReplayCockpit::_unUseSlider);

    register_method("_setPlayBackSpeed", &ReplayCockpit::_setPlayBackSpeed);

    register_method("_setTimestamp", &ReplayCockpit::_setTimestamp);
}

void
avi::gui::ReplayCockpit::_init()
{
}

void
avi::gui::ReplayCockpit::_ready()
{
    playButton = get_node<godot::Button>("Background/Content/Buttons/Play");
    playButton->connect("pressed", this, "_play");

    recordButton = get_node<godot::Button>("Background/Content/Record");
    recordButton->connect("pressed", this, "_record");

    modeButton = get_node<godot::Button>("Background/Content/ChangeMode");
    modeButton->connect("pressed", this, "_enterReplayMode");

    replayName = get_node<godot::LineEdit>("Background/Content/ReplayName");

    slider = get_node<godot::HSlider>("Background/Content/Slider");
    slider->connect("value_changed", this, "_slide");
    slider->connect("mouse_entered", this, "_useSlider");
    slider->connect("mouse_exited", this, "_unUseSlider");

    replayChooser = get_node<godot::OptionButton>("Background/Content/Select/Chooser");
    replayChooser->connect("item_selected", this, "_select");

    refresh = get_node<godot::Button>("Background/Content/Select/Refresh");
    refresh->connect("pressed", this, "_refresh");

    forward1Button = get_node<godot::Button>("Background/Content/Buttons/Forward1");
    forward1Button->connect("pressed", this, "_forward1");

    forward2Button = get_node<godot::Button>("Background/Content/Buttons/Forward2");
    forward2Button->connect("pressed", this, "_forward2");

    rewind1Button = get_node<godot::Button>("Background/Content/Buttons/Rewind1");
    rewind1Button->connect("pressed", this, "_rewind1");

    rewind2Button = get_node<godot::Button>("Background/Content/Buttons/Rewind2");
    rewind2Button->connect("pressed", this, "_rewind2");

    minTimeStamp = get_node<godot::Label>("Background/Content/MinTimestamp/Value");
    maxTimeStamp = get_node<godot::Label>("Background/Content/MaxTimestamp/Value");

    currentTimestamp = get_node<godot::SpinBox>("Background/Content/Progress/Timestamp");
    currentTimestamp->connect("value_changed", this, "_setTimestamp");

    playBackSpeed = get_node<godot::SpinBox>("Background/Content/PlayBack/PlayBackSpeed");
    playBackSpeed->set_value(1);
    playBackSpeed->connect("value_changed", this, "_setPlayBackSpeed");

    this->playButton->set_disabled(true);
    this->forward1Button->set_disabled(true);
    this->forward2Button->set_disabled(true);
    this->rewind1Button->set_disabled(true);
    this->rewind2Button->set_disabled(true);
}

void
avi::gui::ReplayCockpit::setup(avi::replay::ReplayController& controller,
                               avi::scene::VisualizedScene& scene,
                               avi::gui::UserInterface* ui)
{
    this->controller = &controller;
    this->scene = &scene;
    this->ui = ui;
    update_replays();

	if (replayChooser->get_item_count() > 0)
	{
    	_select(0);
	}
}

void
avi::gui::ReplayCockpit::_record()
{
    if (!controller->getRecordingStatus())
    {
        controller->startRecording(getName());
        this->recordButton->set_text("Stop Recording");
    }
    else
    {
        controller->endRecording();
        update_replays();
        this->recordButton->set_text("Start Recording");
    }
}

void
avi::gui::ReplayCockpit::_select(int64_t i)
{
    this->selectedReplay = replayChooser->get_item_text(i).ascii().get_data();
}

void
avi::gui::ReplayCockpit::update_replays()
{
    replayChooser->clear();

    std::vector<std::string> recordingNames = controller->getNames();
    recordingsAvailable = not recordingNames.empty();

    for (auto& name : recordingNames)
    {
        replayChooser->add_item(name.c_str());
    }
}

void
avi::gui::ReplayCockpit::_enterReplayMode()
{
    this->scene->clearScene();
    ui->lockLayerSelection();
    ui->clear();
    if (!controller->inReplayMode())
    {
        ARMARX_INFO << "Entered Replay Mode" << std::endl;
        _select(replayChooser->get_selected_id());
        this->modeButton->set_text("Exit Replay Mode");
        controller->startReplay(selectedReplay);
        this->playButton->set_disabled(false);
        this->forward1Button->set_disabled(false);
        this->forward2Button->set_disabled(false);
        this->rewind1Button->set_disabled(false);
        this->rewind2Button->set_disabled(false);
        this->recordButton->set_disabled(true);

        this->playBackSpeed->set_value(controller->getSpeed());
    }
    else
    {
        ARMARX_INFO << "Exited Replay Mode" << std::endl;
        this->modeButton->set_text("Enter Replay Mode");
        this->playButton->set_text("Play");
        controller->endReplay();
        slider->set_value(0);
        this->playButton->set_disabled(true);
        this->forward1Button->set_disabled(true);
        this->forward2Button->set_disabled(true);
        this->rewind1Button->set_disabled(true);
        this->rewind2Button->set_disabled(true);
        this->recordButton->set_disabled(false);
        this->minTimeStamp->set_text("0");
        this->maxTimeStamp->set_text("0");
        this->currentTimestamp->set_value(0);

        this->playBackSpeed->set_value(controller->getSpeed());
    }
    ui->unlockLayerSelection();
}

void
avi::gui::ReplayCockpit::_forward1()
{
    controller->addSpeed(0.1);
    this->playBackSpeed->set_value(controller->getSpeed());
}

void
avi::gui::ReplayCockpit::_forward2()
{
    controller->addSpeed(0.5);
    this->playBackSpeed->set_value(controller->getSpeed());
}

void
avi::gui::ReplayCockpit::_rewind1()
{
    controller->addSpeed(-0.1);
    this->playBackSpeed->set_value(controller->getSpeed());
}

void
avi::gui::ReplayCockpit::_rewind2()
{
    controller->addSpeed(-0.5);
    this->playBackSpeed->set_value(controller->getSpeed());
}

void
avi::gui::ReplayCockpit::_play()
{
    if (!controller->isPlaying())
    {
        this->playButton->set_text("Pause");
        controller->resumePlayback();
        this->playBackSpeed->set_value(controller->getSpeed());
    }
    else
    {
        this->playButton->set_text("Play");
        controller->pausePlayback();
        this->playBackSpeed->set_value(controller->getSpeed());
    }
}

std::string
avi::gui::ReplayCockpit::getName()
{
    godot::String name = replayName->get_text();
    return name.ascii().get_data();
}

void
avi::gui::ReplayCockpit::_slide()
{
    if (!actualizeSlider) // not true
    {
        if (controller->inReplayMode())
        {
            double value = slider->get_value();
            value =
                value * static_cast<double>(controller->getMaxTime() - controller->getMinTime());
            controller->setTime(static_cast<long>(value));
        }
        else
        {
            this->slider->set_value(0);
        }
    }
}

void
avi::gui::ReplayCockpit::_process(float delta)
{
    if (recordingsAvailable && (!controller->getRecordingStatus()))
    {
        this->modeButton->set_disabled(false);
        this->replayChooser->set_disabled(false);
    }
    else if (!recordingsAvailable || controller->getRecordingStatus())
    {
        this->modeButton->set_disabled(true);
        this->replayChooser->set_disabled(true);
    }

    if (controller->inReplayMode() && !sliderUsed)
    {
        actualizeSlider = true;
        slider->set_value(static_cast<float>(
            static_cast<long double>(controller->getTargetTime()) /
            static_cast<long double>(controller->getMaxTime() - controller->getMinTime())));
        actualizeSlider = false;
    }

    if (controller->inReplayMode())
    {
        this->minTimeStamp->set_text(std::to_string(controller->getMinTime()).c_str());
        this->maxTimeStamp->set_text(std::to_string(controller->getMaxTime()).c_str());
        this->currentTimestamp->set_value(((float)(controller->getTargetTime())) / 1000);
    }
}

void
avi::gui::ReplayCockpit::_useSlider()
{
    sliderUsed = true;
    playingBeforeSlider = controller->isPlaying();
    if (controller->inReplayMode())
    {
        controller->pausePlayback();
    }
}

void
avi::gui::ReplayCockpit::_unUseSlider()
{
    if (playingBeforeSlider)
    {
        controller->resumePlayback();
    }
    sliderUsed = false;
}

void
avi::gui::ReplayCockpit::_setPlayBackSpeed()
{
    if (controller->inReplayMode())
    {
        controller->setSpeed(playBackSpeed->get_value());
    }
    else
    {
        playBackSpeed->set_value(1);
    }
}

void
avi::gui::ReplayCockpit::_setTimestamp()
{
    if (controller->inReplayMode())
    {
        controller->setTime((long)(currentTimestamp->get_value() * 1000));
    }
    else
    {
        currentTimestamp->set_value(0);
    }
}
