#pragma once

#include <Control.hpp>
#include <Godot.hpp>
#include <HSlider.hpp>
#include <Label.hpp>
#include <LineEdit.hpp>
#include <SpinBox.hpp>
#include <arviz_godot/lib/avi/replay/ReplayController.h>
#include <arviz_godot/lib/avi/scene/VisualizedScene.h>

namespace avi::gui
{
    class UserInterface;

    class ReplayCockpit : public godot::Control
    {
        GODOT_CLASS(ReplayCockpit, Control)

    public:
        virtual ~ReplayCockpit() = default;

        static void _register_methods();

        void _init();

        void _ready();

        void setup(avi::replay::ReplayController& controller,
                   avi::scene::VisualizedScene& scene,
                   avi::gui::UserInterface* ui);

        void _process(float delta);

        void _record();

        void _enterReplayMode();

        void _select(int64_t i);

        void _forward1();

        void _forward2();

        void _rewind1();

        void _rewind2();

        void _play();

        void _slide();

        void _useSlider();

        void _unUseSlider();

        void _setPlayBackSpeed();

        void _setTimestamp();

    private:
        void update_replays();

        std::string getName();

        avi::replay::ReplayController* controller{};
        avi::scene::VisualizedScene* scene{};
        avi::gui::UserInterface* ui{};

        bool sliderUsed = false;
        bool playingBeforeSlider = false;
        bool actualizeSlider = false;
        bool recordingsAvailable = false;

        std::string selectedReplay{};

        godot::OptionButton* replayChooser{};
        godot::Button* refresh{};

        godot::Button* recordButton{};

        godot::Button* modeButton;

        godot::Button* playButton{};
        godot::Button* forward1Button{};
        godot::Button* forward2Button{};
        godot::Button* rewind1Button{};
        godot::Button* rewind2Button{};

        godot::LineEdit* replayName{};

        godot::HSlider* slider{};

        godot::Label* minTimeStamp{};
        godot::Label* maxTimeStamp{};

        godot::SpinBox* currentTimestamp{};

        godot::SpinBox* playBackSpeed{};
    };
} // namespace avi::gui
