#include "SettingsWindow.h"

void
avi::gui::SettingsWindow::_register_methods()
{
    register_method("_change", &SettingsWindow::_change);
    register_method("_save", &SettingsWindow::_save);
}

void
avi::gui::SettingsWindow::_init()
{
}

void
avi::gui::SettingsWindow::setup(avi::core::Settings& settings)
{
    this->settings = &settings;

    saveButton = get_node<godot::Button>("Container/Save");
    saveButton->connect("pressed", this, "_save");

    options = settings.getOptions<VisualOptionTransformer>();
    std::vector<avi::core::Option<VisualOptionTransformer>*> toTransform;
    toTransform.reserve(options.size());
    for (auto& option : options)
    {
        toTransform.push_back(option.get());
    }

    VisualOptionTransformer transformer;
    setters = transformer.createGraphicsControls(
        get_node<godot::Control>("Container/Scroll/Content"), this, std::move(toTransform));
}

void
avi::gui::SettingsWindow::open()
{
    setSavedFlag(true);
    this->popup_centered();
}

void
avi::gui::SettingsWindow::_change(godot::Variant v)
{
    for (auto& setter : setters)
    {
        setter();
    }

    setSavedFlag(false);
}

void
avi::gui::SettingsWindow::_save()
{
    setSavedFlag(true);

    settings->save();
}

void
avi::gui::SettingsWindow::setSavedFlag(bool flag)
{
    saveButton->set_text(flag ? "Save" : "Save (*)");
}
