#include "StatusIcons.h"

#include <InputEventMouseButton.hpp>

void
avi::gui::StatusIcons::_register_methods()
{
    godot::register_method("_process", &avi::gui::StatusIcons::_process);
}

void
avi::gui::StatusIcons::_init()
{
}

void
avi::gui::StatusIcons::setup(avi::connection::Connectable& source,
                             avi::connection::StorageModeSwitch& mode,
                             avi::recording::VideoRecorder& recorder)
{
    this->source = &source;
    this->mode = &mode;
    this->recorder = &recorder;

    this->itemList = get_node<godot::ItemList>("Container/List");
    this->disconnectedTexture = this->itemList->get_item_icon(0);
    this->connectedTexture = this->itemList->get_item_icon(1);
    this->replayTexture = this->itemList->get_item_icon(2);
    this->recordingTexture = this->itemList->get_item_icon(3);
    this->replayRecordingTexture = this->itemList->get_item_icon(4);

    this->itemList->clear();

    this->itemList->add_item("Disconnected", this->disconnectedTexture, false);
}

void
avi::gui::StatusIcons::_process(float delta)
{
    if (this->source == nullptr)
    {
        return;
    }

    connectionMode newMode;

    if (mode->isLive())
    {
        if (source->isConnected())
        {
            newMode = CONNECTED;
        }
        else
        {
            newMode = DISCONNECTED;
        }
    }
    else
    {
        newMode = REPLAY;
    }

    bool isRecording = recorder->isRecording();
    bool isRecordingReplay = mode->isRecording();

    if (newMode != this->currentMode || isRecording != this->isVideoRecording ||
        isRecordingReplay != this->isReplayRecording)
    {
        this->currentMode = newMode;
        this->isVideoRecording = isRecording;
        this->isReplayRecording = isRecordingReplay;

        itemList->clear();

        switch (currentMode)
        {
            case DISCONNECTED:
                itemList->add_item("Disconnected", this->disconnectedTexture, false);
                break;
            case CONNECTED:
                itemList->add_item("", this->connectedTexture, false);
                break;
            case REPLAY:
                itemList->add_item("Replay", this->replayTexture, false);
                break;
        }

        if (isVideoRecording)
        {
            itemList->add_item("Recording [V to End]", this->recordingTexture, false);
        }

        if (isReplayRecording)
        {
            itemList->add_item("Recording Replay", this->replayRecordingTexture, false);
        }
    }
}
