#pragma once

#include <Godot.hpp>
#include <Node.hpp>
#include <arviz_godot/lib/avi/connection/InteractionSink.h>
#include <arviz_godot/lib/avi/connection/StorageModeSwitch.h>
#include <arviz_godot/lib/avi/core/InterfaceUtility.h>
#include <arviz_godot/lib/avi/interaction/Manipulator.h>
#include <arviz_godot/lib/avi/scene/CameraSet.h>
#include <arviz_godot/lib/avi/scene/ElementNode.h>

namespace avi::interaction
{

    class InteractionDispatch : public godot::Node
    {
        GODOT_CLASS(InteractionDispatch, Node)

    public:
        virtual ~InteractionDispatch() = default;

        static void _register_methods();

        void _init();

        void _process(float delta);
        void _notification(int what);

        void _unhandled_input(godot::InputEvent* event);

        void setup(avi::connection::InteractionSink& interactionSink,
                   avi::scene::CameraSet& cameraSet,
                   avi::core::InterfaceUtility& ui,
                   avi::connection::StorageModeSwitch& storageMode);

        void sendInteraction(avi::connection::Interaction& interaction);

        void sendManipulation(avi::connection::Interaction& interaction);

        avi::interaction::Manipulator* getManipulator();

    private:
        avi::connection::InteractionSink* interactionSink{};
        avi::connection::StorageModeSwitch* storageMode{};
        avi::scene::CameraSet* cameraSet{};
        avi::core::InterfaceUtility* userInterface{};
        avi::core::Camera* activeCamera;

        std::optional<std::pair<avi::core::Element*, godot::Node*>> targetElement;
        avi::core::Element* previousElement = nullptr;
        avi::connection::Interaction interaction;
        bool wasManipulation = false;

        void deselect();

        std::unique_ptr<avi::interaction::Manipulator> manipulator{};

        void startInteraction();

        void startManipulation();

        bool enableManipulation = true;
    };
} // namespace avi::interaction
