#include "Interaction.h"

#include <Camera.hpp>
#include <Spatial.hpp>
#include <Viewport.hpp>

godot::Interaction
godot::Interaction::compute(const godot::InputEventMouseButton& mouseButtonEvent,
                            const godot::Spatial& node,
                            const Spatial& proxy,
                            const godot::Camera& camera,
                            int axis)
{
    Interaction interaction;

    interaction.original_global = node.get_global_transform();
    interaction.original_local = node.get_transform();
    interaction.center_global = node.get_global_transform().origin;

    interaction.proxy_original_global = proxy.get_global_transform();
    interaction.proxy_original_local = proxy.get_transform();

    godot::Vector2 mouse_pos = node.get_viewport()->get_mouse_position();

    interaction.click_ray_normal_global = get_ray_normal(camera, mouse_pos).normalized();
    interaction.click_ray_origin_global = get_ray_origin(camera, mouse_pos);

    interaction.plane = TransformPlane(TRANSFORM_X_AXIS + axis);

    return interaction;
}

godot::Vector3
godot::Interaction::get_ray_origin(const godot::Camera& camera, const godot::Vector2& p_pos)
{
    return camera.project_ray_origin(p_pos);
}

godot::Vector3
godot::Interaction::get_ray_normal(const godot::Camera& camera, godot::Vector2& p_pos)
{
    return camera.project_ray_normal(p_pos);
}
