#pragma once

#include <Godot.hpp>
#include <Transform.hpp>
#include <Vector2.hpp>
#include <Vector3.hpp>

namespace godot
{
    class Camera;
    class InputEventMouseButton;
    class Spatial;

    enum TransformPlane
    {
        TRANSFORM_X_AXIS,
        TRANSFORM_Y_AXIS,
        TRANSFORM_Z_AXIS
    };

    struct Interaction
    {
        TransformPlane plane;

        Transform original_global;
        Transform original_local;
        Vector3 center_global;

        Vector3 click_ray_normal_global;
        Vector3 click_ray_origin_global;

        godot::Transform proxy_original_global;
        godot::Transform proxy_original_local;

        static Interaction compute(const InputEventMouseButton& mouseButtonEvent,
                                   const Spatial& node,
                                   const Spatial& proxy,
                                   const Camera& camera,
                                   int axis);

        static Vector3 get_ray_origin(const Camera& camera, const Vector2& p_pos);
        static Vector3 get_ray_normal(const Camera& camera, Vector2& p_pos);
    };

} // namespace godot
