#include "ImageRecorder.h"

avi::recording::ImageRecorder::ImageRecorder(scene::CameraSet& cameras,
                                             core::SettingsProvider& settings,
                                             core::EventSubscriptionSystem& events) :
    cameras(cameras), settings(settings)
{
    events.subscribe("TAKE_SCREENSHOT", [this]() { this->takeScreenshot(); });
}

void
avi::recording::ImageRecorder::takeScreenshot()
{
    auto* camera = cameras.getActiveCamera();
    auto screenContent = camera->getCurrentScreenContent();

    std::filesystem::path directory = settings.getScreenshotDirectory();
    std::filesystem::create_directories(directory);

    auto now = std::chrono::system_clock::now();
    std::string timestamp = std::to_string(now.time_since_epoch().count());
    std::string filename = "avi_" + timestamp + ".png";

    std::filesystem::path path = directory / filename;
    screenContent.image->save_png(path.string().c_str());

    godot::OS* os = godot::OS::get_singleton();
    godot::PoolStringArray poolStringArray;
    poolStringArray.push_back("Screenshot saved in");
    poolStringArray.push_back(settings.getScreenshotDirectory().c_str());

    os->execute("notify-send", poolStringArray, false);
}
