#pragma once

#include <chrono>

#include <opencv2/videoio.hpp>

#include <OS.hpp>
#include <arviz_godot/lib/avi/core/EventSubscriptionSystem.h>
#include <arviz_godot/lib/avi/core/SettingsProvider.h>
#include <arviz_godot/lib/avi/scene/CameraSet.h>

namespace avi::recording
{
    class VideoRecorder
    {
    public:
        VideoRecorder(avi::scene::CameraSet& cameras,
                      avi::core::SettingsProvider& settings,
                      avi::core::EventSubscriptionSystem& events);

        void run();

        void startRecording();

        void stopRecording();

        bool isRecording();

    private:
        static const int FPS = 25;
        constexpr static const int64_t MS_PER_FRAME = static_cast<int64_t>(1.0f / FPS * 1000.0f);

        struct Recording
        {
            avi::core::Camera* camera;
            cv::VideoWriter writer;
            cv::Size frameSize;
            std::chrono::steady_clock::time_point lastFrameTime;
        };

        std::optional<Recording> recording;

        avi::scene::CameraSet& cameras;
        avi::core::SettingsProvider& settings;
    };
} // namespace avi::recording
