#include "ReplayController.h"

avi::replay::ReplayController::ReplayController(
    avi::connection::StorageModeSwitch& storageModeSwitch) :
    storageModeSwitch(storageModeSwitch)
{
}

void
avi::replay::ReplayController::startRecording(const std::string& name)
{
    storageModeSwitch.recordReplay(name);
    recording = true;
}

void
avi::replay::ReplayController::endRecording()
{
    storageModeSwitch.endRecording();
    recording = false;
}

std::vector<std::string>
avi::replay::ReplayController::getNames()
{
    return storageModeSwitch.getAllReplays();
}

void
avi::replay::ReplayController::startReplay(const std::string& replay)
{
    clock = std::make_shared<avi::connection::Clock>();
    storageModeSwitch.useReplayTiming(replay, clock);
    replayMode = true;
}

void
avi::replay::ReplayController::endReplay()
{
    storageModeSwitch.useLiveTiming();
    replayMode = false;
    playing = false;
}

void
avi::replay::ReplayController::setTime(long time)
{
    clock->setTargetTime(time);
}

void
avi::replay::ReplayController::setSpeed(float speed)
{
    this->speed = speed;
    if (playing)
    {
        clock->setSpeed(this->speed);
    }
}

long
avi::replay::ReplayController::getTargetTime()
{
    return clock->getTargetTime();
}

void
avi::replay::ReplayController::addSpeed(float add)
{
    this->speed += add;
    if (playing)
    {
        clock->setSpeed(this->speed);
    }
}

long
avi::replay::ReplayController::getMaxTime()
{
    return clock->getTimeFrame().end;
}

long
avi::replay::ReplayController::getMinTime()
{
    return clock->getTimeFrame().begin;
}

float
avi::replay::ReplayController::getSpeed() const
{
    return this->speed;
}

void
avi::replay::ReplayController::pausePlayback()
{
    playing = false;
    clock->setSpeed(0.0);
}

void
avi::replay::ReplayController::resumePlayback()
{
    playing = true;
    if (speed == 0)
    {
        speed = 1.0;
    }
    clock->setSpeed(speed);
}

bool
avi::replay::ReplayController::getRecordingStatus() const
{
    return recording;
}

bool
avi::replay::ReplayController::inReplayMode() const
{
    return replayMode;
}

bool
avi::replay::ReplayController::isPlaying() const
{
    return playing;
}
