#pragma once

#include <RobotAPI/interface/ArViz/Component.h>

#include <OptionButton.hpp>
#include <arviz_godot/lib/avi/connection/StorageModeSwitch.h>

namespace avi::replay
{
    class ReplayController
    {
    public:
        explicit ReplayController(avi::connection::StorageModeSwitch& storageModeSwitch);

        void startRecording(const std::string& name);

        void endRecording();

        std::vector<std::string> getNames();

        void startReplay(const std::string& replay);

        void endReplay();

        void setTime(long time);

        void setSpeed(float speed);

        void addSpeed(float add);

        [[nodiscard]] float getSpeed() const;

        long getTargetTime();

        long getMaxTime();

        long getMinTime();

        void pausePlayback();

        void resumePlayback();

        [[nodiscard]] bool getRecordingStatus() const;

        [[nodiscard]] bool inReplayMode() const;

        [[nodiscard]] bool isPlaying() const;

    private:
        avi::connection::StorageModeSwitch& storageModeSwitch;

        std::shared_ptr<avi::connection::Clock> clock;

        float speed = 1.0;

        bool playing = false;

        bool recording = false;

        bool replayMode = false;
    };
} // namespace avi::replay
