#pragma once

#include "ElementNode.h"
#include <ArrayMesh.hpp>
#include <CollisionShape.hpp>
#include <CylinderMesh.hpp>
#include <CylinderShape.hpp>
#include <Mesh.hpp>
#include <MeshInstance.hpp>
#include <Spatial.hpp>
#include <StaticBody.hpp>

namespace avi::scene
{
    class ArrowCircleElementNode : public avi::scene::ElementNode
    {
        GODOT_CLASS(ArrowCircleElementNode, ElementNode)

    public:
        ~ArrowCircleElementNode() override = default;

        ArrowCircleElementNode();

        void _init();

        static ArrowCircleElementNode*
        newArrowCircleElementNode(avi::connection::SentElementType type,
                                  std::string name,
                                  avi::core::Layer* layer);

        static ArrowCircleElementNode* newArrowCircleElementNode();

        static void _register_methods();

        void _ready();

        godot::AABB getVolume() override;
        godot::Transform getTransform() override;

        void update(IceInternal::Handle<armarx::viz::data::Element> elementInfo) override;

        void setPosition(godot::Vector3 position);

        void setScale(godot::Vector3 scale);

        void setRotation(godot::Quat rotation);

        void setColor(godot::Color color);

        void setCompletion(float completion);

        void setRadius(float radius);

        void setWidth(float width);

        void setShape();

        godot::Vector3 getPosition();

        godot::Vector3 getRotation();

        godot::Vector3 getScale();

        void setAllAttributes(avi::connection::SentElementType type,
                              std::string name,
                              avi::core::Layer* layer);

        void updateAttributes(int flag,
                              std::vector<std::string> interactions,
                              godot::Vector3 position,
                              godot::Vector3 scale,
                              godot::Quat rotation,
                              godot::Color color,
                              float completion,
                              float radius,
                              float width);

    private:
        godot::MeshInstance* torusMeshInstance;
        godot::MeshInstance* cone;
        godot::StaticBody* staticBody;
        godot::CylinderMesh* cylinderMesh;
        godot::CollisionShape* collisionShape;
        float completion;
        float radius;
        float width;
        static const int RINGS = 32;
        static const int SIDES = 8;

        bool shapeChecked = false;

        void generateMesh();

    protected:
        std::vector<godot::Spatial*> getSpatial() override;
    };
} // namespace avi::scene
