#include "CubeElementNode.h"

#include <utility>

#include <BoxShape.hpp>
#include <CubeMesh.hpp>
#include <arviz_godot/lib/avi/core/Utilities.h>
#include <arviz_godot/lib/avi/scene/Converter.h>

void
avi::scene::CubeElementNode::_register_methods()
{
    register_method("_ready", &CubeElementNode::_ready);
}

void
avi::scene::CubeElementNode::_ready()
{
}

void
avi::scene::CubeElementNode::_init()
{
}

void
avi::scene::CubeElementNode::update(IceInternal::Handle<armarx::viz::data::Element> elementInfo)
{
    auto box = IceInternal::Handle<armarx::viz::data::ElementBox>::dynamicCast(elementInfo);

    godot::Vector3 newSize = {box->size.e0, box->size.e1, box->size.e2};
    if (!avi::core::utilities::is_equal_approx(size, newSize))
    {
        size = newSize;
        getMeshInstance()->set_scale(size / 2);
        getCollisionShape()->set_scale(size / 2);
    }

    updateAttributes(box->interaction.enableFlags,
                     box->interaction.contextMenuOptions,
                     avi::scene::Converter::getPosition(box->pose),
                     avi::scene::Converter::getScale(box->scale),
                     avi::scene::Converter::getRotation(box->pose),
                     avi::scene::Converter::getColor(box->color));
}

avi::scene::CubeElementNode*
avi::scene::CubeElementNode::newCubeElementNode(std::string name, avi::core::Layer* layer)
{
    CubeElementNode* node = CubeElementNode::_new();
    godot::Mesh* mesh = godot::CubeMesh::_new();

    node->setStartAttributes(mesh, avi::connection::SentElementType::BOX, std::move(name), layer);
    node->setShape(godot::BoxShape::_new());

    return node;
}
