#include "CylindroidElementNode.h"

#include <CylinderShape.hpp>
#include <arviz_godot/lib/avi/scene/Converter.h>

void
avi::scene::CylindroidElementNode::_register_methods()
{
    register_method("_ready", &CylindroidElementNode::_ready);
}

void
avi::scene::CylindroidElementNode::_ready()
{
}

void
avi::scene::CylindroidElementNode::_init()
{
}

void
avi::scene::CylindroidElementNode::update(
    IceInternal::Handle<armarx::viz::data::Element> elementInfo)
{
    auto cylindroid =
        IceInternal::Handle<armarx::viz::data::ElementCylindroid>::dynamicCast(elementInfo);

    if (!godot::Math::is_equal_approx(height, cylindroid->height))
    {
        height = cylindroid->height;

        getMeshInstance()->set_scale({1, height / 2, 1});
        getCollisionShape()->set_scale({1, height / 2, 1});
    }

    updateAttributes(cylindroid->interaction.enableFlags,
                     cylindroid->interaction.contextMenuOptions,
                     avi::scene::Converter::getPosition(cylindroid->pose),
                     avi::scene::Converter::getScale(cylindroid->scale),
                     avi::scene::Converter::getRotation(cylindroid->pose),
                     avi::scene::Converter::getColor(cylindroid->color));
}

avi::scene::CylindroidElementNode*
avi::scene::CylindroidElementNode::newCylindroidElementNode(std::string name,
                                                            avi::core::Layer* layer)
{
    CylindroidElementNode* node = CylindroidElementNode::_new();
    godot::Mesh* mesh = godot::CylinderMesh::_new();

    node->setStartAttributes(mesh, avi::connection::SentElementType::CYLINDROID, name, layer);
    node->setShape(godot::CylinderShape::_new());

    return node;
}
