#pragma once

#include <memory>

#include <Ice/Handle.h>

#include <RobotAPI/interface/ArViz/Elements.h>

#include <Area.hpp>
#include <Godot.hpp>
#include <MeshInstance.hpp>
#include <Spatial.hpp>
#include <arviz_godot/lib/avi/core/Element.h>

namespace avi::scene
{
    class ElementNode : public godot::Spatial, public avi::core::Element
    {

        GODOT_CLASS(ElementNode, Spatial)

    public:
        ElementNode();
        ~ElementNode() override = default;

        static void _register_methods();

        void _init();

        avi::connection::SentElementType getType() override;

        std::string getName() override;

        avi::core::Layer* getLayer() override;

        int getFlag();

        bool isSelectable() override;

        bool isManipulable() override;

        bool hasContextMenu();

        bool translationX();

        bool translationY();

        bool translationZ();

        bool translationLocal();

        bool rotationX();

        bool rotationY();

        bool rotationZ();

        bool rotationLocal();

        bool scalingX();

        bool scalingY();

        bool scalingZ();

        bool scalingLocal();

        bool transformHide();

        std::vector<std::string> getInteractions() override;

        void setType(avi::connection::SentElementType type);

        void setName(const std::string& name);

        void setLayer(avi::core::Layer* layer);

        void setInteractions(std::vector<std::string> interactions);

        void setInteractionFlag(int flag);

        void setMeshVisible(bool visible);

        godot::AABB getVolume() override = 0;
        godot::Transform getTransform() override = 0;

        void createGhostElement(godot::Transform transform);

        void deselect();

        virtual void update(IceInternal::Handle<armarx::viz::data::Element> elementInfo) = 0;

    private:
        avi::connection::SentElementType type;
        std::string name;
        avi::core::Layer* layer;
        int interactionFlags;
        std::vector<std::string> interactions;

        godot::Spatial* ghostElement{nullptr};

    protected:
        virtual std::vector<godot::Spatial*> getSpatial() = 0;
    };
} // namespace avi::scene
