#include "GraphicsController.h"

#include <utility>

#include <Camera.hpp>
#include <Environment.hpp>

avi::scene::GraphicsController::GraphicsController(godot::Node* root) : root(root)
{
    world = root->get_node<godot::WorldEnvironment>("3D/3D/World");
    viewport3D = root->get_node<godot::Viewport>("3D/3D");
}

void
avi::scene::GraphicsController::activateProfile(avi::scene::Profile profile)
{
    activeProfile = std::move(profile);
    update();
}

avi::scene::Profile&
avi::scene::GraphicsController::getActiveProfile()
{
    return activeProfile;
}

const std::string&
avi::scene::GraphicsController::getActiveProfileName()
{
    return activeProfile.getName();
}

void
avi::scene::GraphicsController::update()
{
    viewport3D->set_msaa(activeProfile.msaa);
    root->get_viewport()->set_msaa(activeProfile.msaa);

    float fov = godot::Math::clamp(static_cast<float>(activeProfile.fov), 1.0f, 179.0f);
    viewport3D->get_camera()->set_fov(fov);
    activeProfile.fov = fov;

    auto environment = world->get_environment();

    environment->set_bg_color(activeProfile.backgroundColor);

    float lightIntensity =
        godot::Math::clamp(static_cast<float>(activeProfile.lightIntensity), 0.0f, 100.0f);
    environment->set_ambient_light_energy(lightIntensity);
    activeProfile.lightIntensity = lightIntensity;

    world->set_environment(environment);

    for (auto& [listener, callback] : changeListeners)
    {
        callback();
    }
}

avi::scene::DrawStyle
avi::scene::GraphicsController::getCurrentDrawStyle() const
{
    return activeProfile.drawStyle;
}

void
avi::scene::GraphicsController::registerChangeListener(void* listener,
                                                       const std::function<void()>& callback)
{
    changeListeners.emplace(reinterpret_cast<uintptr_t>(listener), callback);
}

void
avi::scene::GraphicsController::unregisterChangeListener(void* listener)
{
    auto it = changeListeners.find(reinterpret_cast<uintptr_t>(listener));
    if (it != changeListeners.end())
    {
        changeListeners.erase(it);
    }
}

bool
avi::scene::GraphicsController::isHeadlightEnabled() const
{
    return activeProfile.cameraHeadlight;
}

bool
avi::scene::GraphicsController::isCoordinateSystemEnabled() const
{
    return activeProfile.showCoordinateLines;
}
