#pragma once

#include <functional>
#include <set>

#include <Godot.hpp>
#include <Node.hpp>
#include <Viewport.hpp>
#include <WorldEnvironment.hpp>
#include <arviz_godot/lib/avi/scene/Profile.h>

namespace avi::scene
{
    class GraphicsController
    {
    public:
        explicit GraphicsController(godot::Node* root);

        void activateProfile(avi::scene::Profile profile);

        avi::scene::Profile& getActiveProfile();

        const std::string& getActiveProfileName();

        [[nodiscard]] avi::scene::DrawStyle getCurrentDrawStyle() const;

        [[nodiscard]] bool isHeadlightEnabled() const;
        [[nodiscard]] bool isCoordinateSystemEnabled() const;

        void registerChangeListener(void* listener, const std::function<void()>& callback);
        void unregisterChangeListener(void* listener);

        void update();

    private:
        godot::Node* root;
        godot::Viewport* viewport3D;
        godot::WorldEnvironment* world{};
        avi::scene::Profile activeProfile{};

        std::unordered_map<uintptr_t, std::function<void()>> changeListeners{};
    };

} // namespace avi::scene
