#pragma once

#include "CameraControlScheme.h"
#include <Godot.hpp>
#include <Spatial.hpp>
#include <arviz_godot/lib/avi/scene/ViewCamera.h>
#include <arviz_godot/lib/avi/scene/VisualizedScene.h>

namespace avi::scene
{
    class PlayfulControlScheme : public godot::Spatial, public avi::scene::CameraControlScheme
    {
        GODOT_CLASS(PlayfulControlScheme, Spatial)

    public:
        ~PlayfulControlScheme() override = default;

        static void _register_methods();

        void _init();

        void _ready();

        void _process(float delta);

        void _unhandled_input(godot::InputEvent* event);

    public:
        void setup(avi::core::SettingsProvider& settings);

        void takeOver(avi::scene::ViewCamera* viewCamera) override;

        void release() override;

        void reset() override;

        void focus(godot::AABB box) override;

        void focus(godot::Vector3 point, avi::core::Element* target) override;

        godot::Basis getRotationBasis() override;

    private:
        bool isActive();

        void startTravel(godot::Transform target, float duration);

        void doTravel(float delta);

    private:
        struct Travel
        {
            godot::Transform startTransform{};
            godot::Transform endTransform{};

            float duration{};
            float remainingTime{};
        };

        Travel travel{};
        bool isTraveling{false};

        godot::Spatial* attachment;
        avi::scene::ViewCamera* viewCamera;

        std::vector<bool> inputState;
        std::vector<const char*> inputKeys;

        enum InputStateIndex
        {
            INPUT_FORWARD,
            INPUT_BACKWARD,
            INPUT_LEFT,
            INPUT_RIGHT,
            INPUT_UP,
            INPUT_DOWN,
            INPUT_ROLL_LEFT,
            INPUT_ROLL_RIGHT,
            MAX_INPUT_INDEX
        };

        avi::core::SettingsProvider* settings;

        godot::Vector2 mouseDelta;
        float scrollValue;
    };
} // namespace avi::scene
