#pragma once

#include "ElementNode.h"
#include <ArrayMesh.hpp>
#include <CollisionShape.hpp>
#include <Font.hpp>
#include <Label3D.hpp>
#include <Mesh.hpp>
#include <MeshInstance.hpp>
#include <Shape.hpp>
#include <Spatial.hpp>
#include <StaticBody.hpp>
#include <arviz_godot/lib/avi/core/LanguageUtility.h>

namespace avi::scene
{
    class PoseElementNode : public avi::scene::ElementNode
    {
        GODOT_CLASS(PoseElementNode, ElementNode)

    public:
        PoseElementNode();
        ~PoseElementNode() override = default;

        void _init();

        static PoseElementNode* newPoseElementNode(avi::connection::SentElementType type,
                                                   std::string name,
                                                   avi::core::Layer* layer);

        static PoseElementNode* newPoseElementNode();

        static void _register_methods();

        godot::AABB getVolume() override;
        godot::Transform getTransform() override;

        void update(IceInternal::Handle<armarx::viz::data::Element> elementInfo) override;

        void setPosition(godot::Vector3 position);

        void setScale(godot::Vector3 scale);

        void setRotation(godot::Quat rotation);

        void setColor(godot::Color color);

        godot::Vector3 getPosition();

        godot::Vector3 getRotation();

        godot::Vector3 getScale();

        void setShape();

        void setStartAttributes(avi::connection::SentElementType type,
                                std::string name,
                                avi::core::Layer* layer);

        void updateAttributes(int flag,
                              std::vector<std::string> interactions,
                              godot::Vector3 position,
                              godot::Vector3 scale,
                              godot::Quat rotation,
                              godot::Color color);

        static godot::Mesh* mesh;

    private:
        const real_t textDownscaling = 0.025;

        godot::MeshInstance* poseMeshInstance;
        godot::Label3D* label;

        godot::StaticBody* staticBody;
        godot::CollisionShape* collisionShape;

        bool shapeChecked = false;

    protected:
        std::vector<godot::Spatial*> getSpatial() override;
    };
} // namespace avi::scene
