#pragma once

#include <vector>

#include <Godot.hpp>
#include <Viewport.hpp>
#include <arviz_godot/lib/avi/core/OptionFileReader.h>
#include <arviz_godot/lib/avi/core/OptionFileWriter.h>
#include <arviz_godot/lib/avi/scene/DrawStyle.h>

namespace avi::scene
{
    struct Profile
    {
        godot::Viewport::MSAA msaa = godot::Viewport::MSAA::MSAA_DISABLED;
        godot::Color backgroundColor = godot::Color(1.0, 1.0, 1.0, 1.0);
        double lightIntensity = 1.0f;
        double fov = 70.0f;
        avi::scene::DrawStyle drawStyle = avi::scene::DrawStyle::ORIGINAL;
        bool cameraHeadlight = false;
        bool showCoordinateLines = false;

    public:
        const std::string& getName();

        void rename(const std::string& name);

        void writeToFile(avi::core::OptionFile& optionFile);

        static avi::scene::Profile readFromFile(avi::core::OptionFile& optionFile);

        template <typename T>
        std::vector<std::unique_ptr<avi::core::Option<T>>>
        getOptions()
        {
            std::vector<std::unique_ptr<avi::core::Option<T>>> options;

            options.ADD_OPTION(godot::Viewport::MSAA, msaa, "MSAA");
            options.ADD_OPTION(godot::Color, backgroundColor, "Background Color");
            options.ADD_OPTION(double, lightIntensity, "Light Intensity");
            options.ADD_OPTION(double, fov, "FoV");
            options.ADD_OPTION(avi::scene::DrawStyle, drawStyle, "Draw Style");
            options.ADD_OPTION(bool, cameraHeadlight, "Camera Headlight");
            options.ADD_OPTION(bool, showCoordinateLines, "Coordinate Lines");

            return options;
        }

    private:
        std::string name{};
    };
} // namespace avi::scene
