#pragma once

#include <map>

#include <VirtualRobot/SceneObject.h>
#include <VirtualRobot/VirtualRobot.h>
#include <VirtualRobot/Visualization/CoinVisualization/CoinVisualization.h>
#include <VirtualRobot/XML/RobotIO.h>

#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/system/ArmarXDataPath.h>
#include <ArmarXCore/core/system/cmake/CMakePackageFinder.h>

#include <RobotAPI/interface/ArViz/Elements.h>

#include "ElementNode.h"
#include <Godot.hpp>
#include <arviz_godot/lib/avi/core/ModelLoader.h>
#include <arviz_godot/lib/avi/gui/GraphicsSettings.h>
#include <arviz_godot/lib/avi/scene/DrawStyle.h>
#include <arviz_godot/lib/avi/scene/GraphicsController.h>
#include <arviz_godot/lib/avi/scene/MeshElementNode.h>
#include <arviz_godot/lib/avi/scene/ObjectElementNode.h>
#include <arviz_godot/lib/avi/scene/PoseElementNode.h>

namespace avi::scene
{
    class RobotElementNode : public avi::scene::ElementNode
    {
        GODOT_CLASS(RobotElementNode, ElementNode)

    public:
        void _init();
        static void _register_methods();
        RobotElementNode();
        ~RobotElementNode() override = default;

        godot::AABB getVolume() override;
        godot::Transform getTransform() override;

        void update(IceInternal::Handle<armarx::viz::data::Element> elementInfo) override;

        void setPosition(godot::Vector3 position);

        void setScale(godot::Vector3 scale);

        void setRotation(godot::Quat rotation);

        void addShape(godot::Mesh* mesh, const std::string& nodeName);

        void setStartAttributes(avi::scene::GraphicsController* controller,
                                avi::core::ModelLoader* modelLoader,
                                avi::connection::SentElementType type,
                                std::string name,
                                avi::core::Layer* layer,
                                int flag,
                                std::vector<std::string> interactions,
                                godot::Vector3 position,
                                godot::Vector3 scale,
                                godot::Quat rotation,
                                godot::Color color,
                                std::string project,
                                std::string filename,
                                int drawStyle,
                                std::map<std::string, float> jointValues);

        void updateAttributes(int flag,
                              std::vector<std::string> interactions,
                              godot::Vector3 position,
                              godot::Vector3 scale,
                              godot::Quat rotation,
                              godot::Color color,
                              std::string project,
                              std::string filename,
                              int drawStyle,
                              const std::map<std::string, float>& jointValues);

        VirtualRobot::RobotPtr loadRobot(std::string project, std::string filename);

        void updateVisualization();
        void visualizeRobot();

        static RobotElementNode* newRobotElementNode();

        void visualizationChanged();

        void _notification(int what);

        void setOverrideColor(godot::Color color, int drawStyle);

    private:
        class RobotCache
        {
            using Key = std::pair<std::string, std::string>;

            struct Entry
            {
                Key key;
                VirtualRobot::RobotPtr value;
            };

        public:
            std::optional<VirtualRobot::RobotPtr> get(const std::string& project,
                                                      const std::string& filename);
            void set(const std::string& project,
                     const std::string& filename,
                     VirtualRobot::RobotPtr robot);

        private:
            std::vector<Entry> entries;
        };

        static RobotCache& getCache();

    private:
        avi::core::ModelLoader* modelLoader;
        std::string project;
        std::string filename;
        VirtualRobot::RobotPtr robot;

        int armarXDrawStyle = -1;
        avi::scene::DrawStyle drawStyle = avi::scene::DrawStyle::VISUALIZATION;
        godot::Color color;
        godot::StaticBody* staticBody;

        std::map<std::string, godot::MeshInstance*> nodes;
        std::map<std::string, godot::CollisionShape*> collisionShapes;

        avi::scene::GraphicsController* graphicsController;

        bool shapeChecked = false;
        bool isOverrideDirty = true;

    protected:
        std::vector<godot::Spatial*> getSpatial() override;
    };

} // namespace avi::scene
