#pragma once

#include <Godot.hpp>
#include <Spatial.hpp>
#include <arviz_godot/lib/avi/scene/CameraControlScheme.h>
#include <arviz_godot/lib/avi/scene/VisualizedScene.h>

namespace avi::scene
{
    class ViewCamera;

    class TechnicalControlScheme : public godot::Spatial, public avi::scene::CameraControlScheme
    {
        GODOT_CLASS(TechnicalControlScheme, Spatial)

    public:
        ~TechnicalControlScheme() override = default;
        static void _register_methods();

        void _init();

        void _ready();

        void _unhandled_input(godot::InputEvent* event);

        void _process(float delta);

    public:
        void setup(avi::core::SettingsProvider& settings);

        void takeOver(avi::scene::ViewCamera* viewCamera) override;

        void release() override;

        void reset() override;

        void focus(godot::AABB box) override;

        void focus(godot::Vector3 point, avi::core::Element* target) override;

        godot::Basis getRotationBasis() override;

    private:
        bool isActive();

        void orbit(godot::Vector2 rotation, float deltaTime);

        void move(godot::Vector2 movement, float deltaTime);

        void zoom(float delta, float deltaTime);
        static float getClampedZoom(float zoom);

        void roll(float delta, float deltaTime);

        void startTravel(godot::Transform anchor, float zoom, float duration);

        void doTravel(float deltaTime);

    private:
        struct Travel
        {
            godot::Transform startTransform{};
            godot::Transform endTransform{};

            float startZoom{};
            float endZoom{};

            float duration{};
            float remainingTime{};
        };

        godot::Vector2 mouseDelta;
        float wheelDelta;
        float smoothWheelDelta;

        float zoomValue;
        float zoomDistance;

        godot::Spatial* anchor;
        godot::Spatial* zoomer;

        godot::Spatial* attachment;

        avi::scene::ViewCamera* viewCamera;

        avi::core::SettingsProvider* settings;

        Travel travel;
        bool isTraveling{};
    };
} // namespace avi::scene
