#pragma once

#include "ElementNode.h"
#include <CollisionShape.hpp>
#include <Font.hpp>
#include <Label3D.hpp>
#include <PackedScene.hpp>
#include <ResourceLoader.hpp>
#include <StaticBody.hpp>
#include <ViewportTexture.hpp>
#include <arviz_godot/lib/avi/core/LanguageUtility.h>

namespace avi::scene
{
    class TextElementNode : public avi::scene::ElementNode
    {
        GODOT_CLASS(TextElementNode, ElementNode)

    public:
        void _init();

        static TextElementNode* newTextElementNode(godot::Spatial* scene);

        TextElementNode();
        ~TextElementNode() override = default;

        godot::AABB getVolume() override;
        godot::Transform getTransform() override;

        void update(IceInternal::Handle<armarx::viz::data::Element> elementInfo) override;

        void setPosition(godot::Vector3 position);

        void setScale(godot::Vector3 scale);

        void setRotation(godot::Quat rotation);

        void setColor(godot::Color color);

        void setText(const std::string& newText);

        void setShape();

        godot::Vector3 getPosition();

        godot::Vector3 getRotation();

        godot::Vector3 getScale();

        void updateAttributes(int flag,
                              std::vector<std::string> interactions,
                              godot::Vector3 position,
                              godot::Vector3 scale,
                              godot::Quat rotation,
                              godot::Color color,
                              std::string text);

        void setStartAttributes(avi::connection::SentElementType type,
                                std::string name,
                                avi::core::Layer* layer);

    private:
        static godot::Font* font;

        godot::StaticBody* staticBody;
        godot::CollisionShape* collisionShape;

        godot::Label3D* label;

        static float scalingFactor;

        std::string text;

        bool shapeChecked = false;

    protected:
        std::vector<godot::Spatial*> getSpatial() override;
    };
} // namespace avi::scene
